#include "bitmap.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <string.h>

/* bitmap class for C.

   Copyright (C) 2000 Danny Yoo (dyoo@hkn.eecs.berkeley.edu)

   Your basic bitmap class.  Nothing really special, except I went a
   little overboard and made it look very class-like.  */

struct BitmapImpl {
  unsigned int* data;
  long size;
};

/* We will use an array of unsigned integers.  I'll assume that an
   unsigned int has 32 bits... later, I'll rewrite this to be less
   platform dependent. */
/* A few convenient constants to work with. */
#define INT_BITSIZE 32
#define ALL_ONES 0xffffffff
#define ALL_ZEROS 0x00000000
#define FIRST_BIT 0x00000001

/* Given an index into the bitmap, we'd like an index in respect
   to a char (byte).  */
static long getIndex(long bitIndex) {
  return bitIndex / INT_BITSIZE;
}
/* Given an index into the bitmap, we'd like the char offset. */
static int getOffset(long bitIndex) {
  return bitIndex % INT_BITSIZE;
}

Bitmap Bitmap_new(long size) {
  Bitmap b;
  long num_ints;

  num_ints = getIndex(size-1) + 1;
  b = malloc(sizeof(struct BitmapImpl));
  b->data = calloc(sizeof(unsigned int), num_ints);
  b->size = size;
  return b;
}

void Bitmap_delete(Bitmap map) {
  free(map->data);
  free(map);
}

void Bitmap_set(Bitmap map, long n) {
  long index;
  int offset;
  assert(0 <= n && n < map->size);
  index = getIndex(n);
  offset = getOffset(n);
  *(map->data + index) |= (FIRST_BIT << offset);
}

void Bitmap_unset(Bitmap map, long n) {
  long index;
  int offset;
  assert(0 <= n && n < map->size);
  index = getIndex(n);
  offset = getOffset(n);
  *(map->data + index) &= ~(FIRST_BIT << offset);
}


int Bitmap_get(Bitmap map, long n) {
  long index;
  int offset;
  assert(0 <= n && n < map->size);
  index = getIndex(n);
  offset = getOffset(n);
  return (*(map->data + index) & (FIRST_BIT << offset)) != 0;
}

long Bitmap_getSize(Bitmap map) {
  return map->size;
}
